/*
 *      $Filename: banfile.c $
 *      $Revision: 3.18 $
 *      $Date: 1996/06/28 19:58:37 $
 *
 *      Copyright (C) 1994-96 by Peter Simons <simons@peti.rhein.de>
 *
 *      This program is free software; you can redistribute it and/or
 *      modify it under the terms of the GNU General Public License as
 *      published by the Free Software Foundation; either version 2 of
 *      the License, or (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *      General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *      $Id: banfile.c,v 3.18 1996/06/28 19:58:37 simons Exp $
 *
 */

/************************************* includes ***********/
#include "petidomo.h"
#include <stdio.h>
#include <stdlib.h>
#if defined(HAVE_REGEX_H) && defined(HAVE_REGCOMP) && defined(HAVE_REGEXEC)
#include <regex.h>
#else
#include "regex.h"
#endif

/************************************* defines ************/

/************************************* prototypes *********/

/************************************* global variables ***/
static char rcssid[] = "$Id: banfile.c,v 3.18 1996/06/28 19:58:37 simons Exp $";

/************************************* program ************/
int 
IsBanned(char *address, char *listname)
{
    FILE     *banfile;
    char      banfilename[1024],
              pattern[256];
    int       banned = 0,
              not;
    regex_t   preg;
    int       result = 0;
    int       need_to_free = 0;

    debug(2, "isBanned(%s, %s)", address, listname);

    if (listname && *listname)
	sprintf(banfilename, "%s/%s/banned", LISTDIR, listname);
    else
	sprintf(banfilename, "%s/banned", SERVDIR);

    if ((banfile = fopen(banfilename, "r"))) {
	while (fgets(pattern, 1024 - 1, banfile)) {
	    /* remove the trailig newline */
	    pattern[strlen(pattern) - 1] = '\0';

	    /* Do we have to invert the result (NOT)? */
	    if (pattern[0] == '!' && pattern[1] == ' ')
		not = 1;
	    else
		not = 0;
	    if (!regcomp(&preg, (not) ? pattern + 2 : pattern, REG_EXTENDED | REG_ICASE)) {
		result = regexec(&preg, address, (int) NULL, NULL, 0) != REG_NOMATCH;
		need_to_free++;
	    }
	    if (not)
		result = !result;
	    if (result) {
		debug(3, "Matching %s against %s: HIT.", address, pattern);
		banned = 1;
		break;
	    }
	    else
		debug(3, "Matching %s against %s: NOTHIN'", address, pattern);
	}
	fclose(banfile);
    }

    if (need_to_free > 0) {
	regfree(&preg);
	debug(3, "banfile was NOT empty!");
    }
    else
	debug(3, "banfile was empty!");

    return banned;
}
