/*
 *      $Filename: listmail.c $
 *      $Revision: 1.33 $
 *      $Date: 1996/11/10 00:16:49 $
 *
 *      Copyright (C) 1995-96 by Peter Simons <simons@peti.rhein.de>
 *
 *      This program is free software; you can redistribute it and/or
 *      modify it under the terms of the GNU General Public License as
 *      published by the Free Software Foundation; either version 2 of
 *      the License, or (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *      General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *      $Id: listmail.c,v 1.33 1996/11/10 00:16:49 simons Exp $
 *
 */

/************************************* includes ***********/
#include "petidomo.h"

/************************************* defines ************/

/************************************* prototypes *********/
char     *FilterArticle(const char *article);
char     *FindFromAddress(char *article);
char     *FindXReplyTo(char *article);
char     *FindApproved(char *article);
unsigned int LineLen(const char *);

/************************************* global variables ***/
static char rcssid[] = "$Id: listmail.c,v 1.33 1996/11/10 00:16:49 simons Exp $";

/************************************* subroutines ********/
int
ListMail(char *listname, int isInsertListname)
{
    FILE     *fh;
    int       fd;
    char      buf[BUFSIZE],
             *buffer,
             *header,
             *origheader,
             *body,
             *replyto,
             *passwd;
    int       len = 0;
    int       buffersize = 2 * 1024;

    lowercase(listname);	/* We don't want case in here. */
    openlog("listmail", LOG_CONS | LOG_PID, LOG_MAIL);
    setdebuglevel(0);

    /* Read header from standard input, so that we have the relevant header
       lines available easily. */

    debug(2, "Read mail header from standard input.");
    buffer = xmalloc(buffersize);
    while (fgets(buffer + len, buffersize - len - 1, stdin) && buffer[len] != '\n') {
	debug(3, "READ: %s", buffer + len);
	len += strlen(buffer + len);
	if (buffersize - len <= 80) {
	    buffersize += 1024;	/* Buffer overflow */
	    if (!(buffer = realloc(buffer, buffersize))) {
		log(LOG_ERR, "Unable to allocate %d byte.", buffersize);
		exit(10);
	    }
	}
    }
    if (buffer[len] == '\n') {
	buffer[len] = '\0';
    }
    else {
	log(LOG_WARNING, "Received garbled message for list '%s'.", listname);
	return 0;
    }

    /* Get X-Reply-To: if set. */
    debug(2, "Determining whether Reply-To should be overwritten.");
    replyto = FindXReplyTo(buffer);
    debug(2, "X-Reply-To was set to \"%s\".", (replyto) ? replyto : "none");

    /* Get Approve[d]: if set. */
    debug(2, "Determining whether a password has been provided.");
    passwd = FindApproved(buffer);
    if (passwd == NULL)
	passwd = "";
    debug(2, "Approved was set to \"%s\".", (passwd) ? passwd : "none");

    /* Filter out the header lines we want. */

    debug(2, "filter our the header lines we want.");
    if (!(header = FilterArticle(buffer))) {
	log(LOG_ERR, "Internal error: FilterArticle() failed.");
	exit(10);
    }
    origheader = xstrdup(buffer);
    free(buffer);

    /* Read the message body. */

    debug(2, "Read mail body from standard input.");
    buffersize = 8 * 1024;
    len = 0;
    buffer = xmalloc(buffersize);

    while (fgets(buffer + len, buffersize - len - 1, stdin)) {
	debug(3, "READ: %s", buffer + len);
	len += strlen(buffer + len);
	if (buffersize - len <= 80) {
	    buffersize += 4 * 1024;	/* Buffer overflow */
	    if (!(buffer = realloc(buffer, buffersize))) {
		log(LOG_ERR, "Unable to allocate %d byte.", buffersize);
		exit(10);
	    }
	}
    }
    if (len) {
	if (buffer[len - 1] != '\n')	/* Make sure, the last line has a
					   trailing \n character. */
	    buffer[len++] = '\n';
	buffer[len] = '\0';
    }
    else {
	log(LOG_WARNING, "Received empty article for list '%s'.", listname);
	return 0;
    }
    body = xstrdup(buffer);
    free(buffer);

    /* Extract the address of the sender and do pattern matching and other
       stuff to recognize banned addresses, etc... */

    debug(2, "Determining, whether address is banned from posting.");
    buffer = FindFromAddress(header);
    debug(2, "Article came from \"%s\".", (buffer) ? buffer : "none");
    if (IsBanned(buffer, NULL) || IsBanned(buffer, listname)) {
	log(LOG_INFO, "Rejected posting from banned address '%s' to list '%s'.",
	    buffer, listname);
	sprintf(buf, "%s owner-%s", SENDMAIL, listname);
	if (!(mailer = popen(buf, MODE_PIPETO))) {
	    log(LOG_ERR, "Can't open sendmail for mailhandling!");
	    exit(10);
	}
	fprintf(mailer, "To: owner-%s@%s.%s\n", listname, NODENAME, DOMAINNAME);
	fprintf(mailer, "Subject: Rejected article from list '%s'\n", listname);
	fprintf(mailer, "\nReason: Address '%s' is banned from posting.\n\n", buffer);
	fprintf(mailer, origheader);
	fprintf(mailer, "\n");
	fprintf(mailer, "%s", body);
	return 0;
    }
    debug(2, "Check whether a required password for posting has been provided.");
    if (!posting_authen(listname, passwd)) {
	log(LOG_INFO, "Rejected posting from address '%s' to list '%s': No valid password!",
	    buffer, listname);
	sprintf(buf, "%s owner-%s", SENDMAIL, listname);
	if (!(mailer = popen(buf, MODE_PIPETO))) {
	    log(LOG_ERR, "Can't open sendmail for mailhandling!");
	    exit(10);
	}
	fprintf(mailer, "To: owner-%s@%s.%s\n", listname, NODENAME, DOMAINNAME);
	fprintf(mailer, "Subject: Rejected article from list '%s'\n", listname);
	fprintf(mailer, "\nReason: Poster failed to provide a valid password.\n\n");
	fprintf(mailer, origheader);
	fprintf(mailer, "\n");
	fprintf(mailer, "%s", body);
	return 0;
    }

    log(LOG_INFO, "Posting article from '%s' to list '%s'.", buffer, listname);
    free(buffer);

    /* Find correct Reply-To: address. */
    debug(2, "Determine the Reply-To to use.");
    if (replyto == NULL) {
	sprintf(buf, "%s/%s/reply-to", LISTDIR, listname);
	fh = fopen(buf, "r");
	if (fh) {
	    if (fgets(buf, sizeof(buf), fh)) {
		replyto = xstrdup(buf);
	    }
	    fclose(fh);
	}
	if (replyto == NULL) {
	    sprintf(buf, "%s@%s.%s\n", listname, NODENAME, DOMAINNAME);
	    replyto = xstrdup(buf);
	}
    }
    else
	debug(2, "Reply-To was set via X-Reply-To.");
    debug(2, "Final Reply-To is \"%s\".", replyto);

    /* If -v has been set, insert the list's name in the subject line. */

    if (isInsertListname) {
	char *    tmp_buf;
	char *    buf;
	char *    subjectLine;

	buf = header;
	while ((subjectLine = strstr(buf, "Subject:")) != NULL) {
	    if (subjectLine == header || subjectLine[-1] == '\n')
	      break;
	    else
	      buf = subjectLine;
	}
	if (subjectLine) {
	    /* Test whether the subject contains the the listname
               already. */
	    tmp_buf = xmalloc(LineLen(subjectLine)+1);
	    strncpy(tmp_buf, subjectLine, LineLen(subjectLine));
 	    if (strstr(tmp_buf, listname) == NULL) {
		/* No, it doesn't. So append it. */
		buf = xmalloc(strlen(header) + strlen(listname) + 3); /* '[listname] ' */
		strncpy(buf, header, ((subjectLine - header) + strlen("Subject:")));
		strcat(buf, " [");
		strcat(buf, listname);
		strcat(buf, "]");
		strcat(buf, header+((subjectLine - header) + strlen("Subject:")));
		free(header);
		header = buf;
	    }
	    free(tmp_buf);
	}
	else {
	    buf = xmalloc(strlen(header) + strlen(listname) + 13); /* 'Subject: [listname]' */
	    sprintf(buf, "%sSubject: [%s]\n", header, listname);
	    free(header);
	    header = buf;
	}
    }

    /* Open pipe to sendmail and give him the mail we just received with
       filtered headers. */

    sprintf(buf, "%s -f owner-%s %s-out", SENDMAIL, listname, listname);
    debug(1, "Spawning sendmail: \"%s\".", buf);
    if (!(mailer = popen(buf, MODE_PIPETO))) {
	logerrno(LOG_ERR, "popen(%s) failed", buf);
	log(LOG_ERR, "Can't open sendmail for mailhandling!");
	exit(10);
    }
    fprintf(mailer, "%s", header);
    fprintf(mailer, "Precedence: list\n");
    fprintf(mailer, "Sender: owner-%s@%s.%s\n", listname, NODENAME, DOMAINNAME);
    fprintf(mailer, "To: %s@%s.%s\n", listname, NODENAME, DOMAINNAME);
    if (!strcasecmp(replyto, "none\n"))
	debug(2, "No Reply-to: will be set at all.");
    else
	fprintf(mailer, "Reply-To: %s", replyto);
    fprintf(mailer, "\n");
    fprintf(mailer, "%s", body);

    /* Append the list's signature, if available. */

    debug(2, "Appending the list's signature if available.");
    sprintf(buf, "%s/%s/signature", LISTDIR, listname);
    debug(3, "Opening \"%s\".", buf);
    if ((fd = open(buf, O_RDONLY, 0)) != -1) {
	while ((len = read(fd, buf, BUFSIZE - 1)) && len != -1) {
	    fwrite(buf, 1, len, mailer);
	}
	close(fd);
	debug(2, "Signature appended.");
    }
    else
	debug(2, "No signature available.");

    pclose(mailer);

    /* Log the posted article. */

    debug(2, "Append the article to the archive.");
    sprintf(buf, "%s/%s/log", LISTDIR, listname);
    debug(3, "Opening file \"%s\".", buf);
    if ((fd = open(buf, O_WRONLY | O_APPEND, 0)) != -1) {
	write(fd, origheader, strlen(origheader));
	write(fd, "\n", 1);
	write(fd, body, strlen(body));
	close(fd);
    }
    else
	debug(3, "No archive wanted...");

    return 0;
}

 /* This routine will determine the length of a header line, which must be
    terminated by either a newline or a NUL byte. Entries which continue at the
    next line are recognized correctly. The last \n or \0 is included in the
    number of characters. */

unsigned int
LineLen(const char *line)
{
    int       i = 0;

    while (1) {
	while (line[i] != '\n' && line[i] != '\0')
	    i++;
	if (line[i] == '\0' || !(isspace(line[i + 1])))
	    return i + 1;	/* include the trailing \n or \0 */
	i++;
    }
}

 /* isHeader() tests, whether the given line is the keyword header
    specified by the parameter. Valid keywords must have a terminating
    colon (':'). */

int
isHeader(const char *line, const char *keyword)
{
    int       len = strlen(keyword);

    if (strncasecmp(line, keyword, len))
	return 0;

    return 1;
}

 /* This routine will expect a buffer holding a mailheader. Then it will filter
    out all relevant header lines and copy them to a new buffer. This buffer is
    allocated with malloc() and should be freed immediately, when not required
    anymore by the main program. */

char     *
FilterArticle(const char *article)
{
    char     *ptr,
             *buffer;

    /* I know, this shouldn't happen. But better test here, than later when the
       core dump comes. */
    if (!article || !(*article)) {
	log(LOG_ERR, "Internal Error: Got empty article!");
	return NULL;
    }

    /* Allocate internal buffers. */
    if (!(buffer = malloc(strlen(article)))) {
	log(LOG_ERR, "Unable to allocate %d byte.", strlen(article));
	return NULL;
    }

    /* Parse article for certain headers and copy them to the buffer. Also copy
       the article body unmodified. */
    for (ptr = buffer; *article != '\n' && *article != '\0';) {
	int       len;

	if (isHeader(article, "Subject:") ||
	    isHeader(article, "From:") ||
	    isHeader(article, "Date:") ||
	    isHeader(article, "MIME-Version:") ||
	    isHeader(article, "Content-Type:") ||
	    isHeader(article, "Content-Transfer-Encoding:") ||
	    isHeader(article, "In-Reply-To:") ||
	    isHeader(article, "Message-Id:")) {
	    len = LineLen(article);
	    strncpy(ptr, article, len);
	    article += len;
	    ptr += len;
	}
	else {
	    while (*article != '\n' && *article != '\0')
		article++;
	    if (*article)
		article++;
	}
    }

    *ptr = '\0';

    return buffer;
}

/* This routine searches through a buffer and finds the From: header
   line. Then it extracts the actual address and returns it in an
   mallocated buffer -- which should be freed as soon as possible. */

char     *
FindFromAddress(char *article)
{
    char     *buffer = article;
    int       i;

    while (*buffer) {
	while (strncasecmp(buffer, "From:", 5) && *buffer) {
	    buffer++;
	}
	if (buffer == article || buffer[-1] == '\n')
	    break;
    }
    if (*buffer) {
	buffer += 5;		/* skip the 'From:' part */
	i = 0;
	while (buffer[i] && buffer[i] != '\n')
	    i++;
	if ((article = malloc(i + 1))) {
	    strncpy(article, buffer, i);
	    article[i] = '\0';
	    ParseAddress(article);
	    return article;
	}
    }
    else {
	log(LOG_WARNING, "Received mail without From: line.");
	exit(0);
    }
    log(LOG_ERR, "Couldn't extract address from From: line.");
    exit(10);
}

/* This routine searches through a buffer and finds the X-Reply-To:
   header line. Then it extracts the actual address and returns it in
   an mallocated buffer -- which should be freed as soon as possible. */

char     *
FindXReplyTo(char *article)
{
    char     *buffer = article;
    int       i;

    while (*buffer) {
	while (strncasecmp(buffer, "X-Reply-To:", 11) && *buffer)
	    buffer++;

	if (buffer == article || buffer[-1] == '\n')
	    break;
	else
	    buffer++;
    }

    if (*buffer) {

	/* skip the 'X-Reply-To:' part */
	buffer += 11;

	/* skip prefacing whitespace */
	while (isspace(*buffer))
	    buffer++;

	/* Determine length of string. */
	for (i = 0; buffer[i] && buffer[i] != '\n'; i++) ;

	/* Copy string into new buffer. */

	if ((article = malloc(i + 2))) {
	    strncpy(article, buffer, i + 1);	/* include the trailing return */
	    article[i + 1] = '\0';
	    return article;
	}
    }

    return NULL;
}

char     *
FindApproved(char *article)
{
    char     *buffer = article;
    int       i;

    while (*buffer) {
	while (*buffer &&
	       strncasecmp(buffer, "Approved:", 9) && strncasecmp(buffer, "Approve:", 8))
	    buffer++;

	if (buffer == article || buffer[-1] == '\n')
	    break;
	else
	    buffer++;
    }

    if (*buffer) {

	/* skip the 'Approve[d]:' part */
	if (!strncasecmp(buffer, "Approved:", 9))
	    buffer += 9;
	else
	    buffer += 8;

	/* skip prefacing whitespace */
	while (isspace(*buffer))
	    buffer++;

	/* Determine length of string. */
	for (i = 0; buffer[i] && buffer[i] != '\n'; i++) ;

	/* Strip a trailing return. */
	if (buffer[i] == '\n')
	    i--;

	/* Copy string into new buffer. */

	if ((article = malloc(i + 2))) {
	    strncpy(article, buffer, i + 1);
	    article[i + 1] = '\0';
	    return article;
	}
    }

    return NULL;
}
