/*
 *      $Filename: petidomo.c $
 *      $Revision: 3.13 $
 *      $Date: 1996/12/23 00:20:17 $
 *
 *      Copyright (C) 1993 by Brian Kantor <brian@nothing.ucsd.edu>
 *      Copyright (C) 1994-96 by Peter Simons <simons@peti.rhein.de>
 *
 *      This program is free software; you can redistribute it and/or
 *      modify it under the terms of the GNU General Public License as
 *      published by the Free Software Foundation; either version 2 of
 *      the License, or (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *      General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *      $Id: petidomo.c,v 3.13 1996/12/23 00:20:17 simons Exp $
 *
 */

/************************************* includes ***********/
#include "petidomo.h"
#include "version.h"

/************************************* defines ************/
#define LS_USAGE   "USAGE: petidomo <mailfile [default_list]\n"
#define LM_USAGE   "USAGE: listmail <mailfile [-v] listname\n"

/************************************* prototypes *********/
int       isKeyword(char *, char *);
void      ParseAddress(char *);

/************************************* global variables ***/
static char rcssid[] = "$Id: petidomo.c,v 3.13 1996/12/23 00:20:17 simons Exp $";
FILE     *mailer;
char     *def_listname,
         *tmpbuf,
          passwd[81];
char      NODENAME[MAXHOSTNAMELEN],
          DOMAINNAME[MAXHOSTNAMELEN];

/************************************* program ************/
int
main(int argc, char **argv)
{
    char     *from = NULL, *replyto = NULL,
             *buf,
             *grp;
    int       gotcommand = 0,
              garbage = 0;
    int       rc;

    /*
     * Determine fully qualified domain name of the host.
     */
#if defined(HAVE_GETHOSTNAME) || defined(HAVE_UNAME)
#  ifdef HAVE_GETHOSTNAME
    rc = gethostname(NODENAME, sizeof(NODENAME));
    if (rc == -1) {
	fprintf(stderr, "%s: gethostname() failed: Can't determine my fully qualified domain name\n", argv[0]);
	return 10;
    }
#  else
    {
	struct utsname utsname;
	rc = uname(&utsname);
	if (rc == -1) {
	    fprintf(stderr, "%s: gethostname() failed: Can't determine my fully qualified domain name\n", argv[0]);
	    return 10;
	}
	strcpy(NODENAME, utsname.nodename);
    }
#  endif
    buf = strchr(NODENAME, (int) '.');
    if (buf == NULL) {
#  if defined(HAVE_GETHOSTBYNAME) || defined(HAVE_LIBNSL)
	struct hostent * hent;

	hent = gethostbyname(NODENAME);
	if (hent == NULL) {
	    fprintf(stderr, "%s: gethostbyname() failed: Can't determine my fully qualified domain name\n", argv[0]);
	    return 10;
	}
	else {
	    buf = strchr(hent->h_name, (int) '.');
	    if (buf == NULL) {
		fprintf(stderr, "%s: gethostbyname() did not return the fully qualified domain name.\n", argv[0]);
		return 10;
	    }
	    else {
		buf++;
		strcpy(DOMAINNAME, buf);
	    }
	}
#  else
	/* We don't have gethostbymame() -- so give up. */
	fprintf(stderr, "%s: gethostname() failed: Can't determine my fully qualified domain name\n", argv[0]);
	return 10;
#  endif
    }
    else {
	*buf++ = '\0';
	strcpy(DOMAINNAME, buf);
    }
#else
#  if !defined(FQDN)
#    error "Please define your fully qualified domainname in petidomo.h. You'll find instructions what to do there."
#  endif
    /* Us a hard-coded fully-qualified domainname. */
    {
	int       i;

	strcpy(NODENAME, FQDN);
	for (i = 0; NODENAME[i] != '.'; i++) ;
	NODENAME[i] = '\0';
	strcpy(DOMAINNAME, NODENAME + i + 1);
    }
#endif

    /*
     * Determine the program name we have been called under and jump into
     * listmail if appropriate.
     */
    if (!strcasecmp(argv[0] + (strlen(argv[0]) - strlen("listmail")), "listmail")) {
	/* skip the path in the argv[0] */
	switch (argc) {
	  case 2:
	      if (!strcmp(argv[1], "-v")) {
		  fprintf(stderr, LM_USAGE);
		  return 20;
	      }
	      else
		return ListMail(argv[1], 0);
	  case 3:
	      if (!strcmp(argv[1], "-v"))
		return ListMail(argv[2], 1);
	  default:
	    fprintf(stderr, LM_USAGE);
	    return 20;
	}
    }

    if (argc > 2 || (argc == 2 && (!strcmp(argv[1], "?") || !strcasecmp(argv[1], "-h")))) {
	fprintf(stderr, LS_USAGE);
	return 20;
    }

    /*
     * Init various stuff.
     */
    openlog("petidomo", LOG_CONS | LOG_PID, LOG_MAIL);
    setdebuglevel(0);
    umask(0117);

    /*
     * Allocate our internal buffers.
     */
    tmpbuf = xmalloc(BUFSIZE);
    buf = xmalloc(BUFSIZE);
    grp = xmalloc(64);

    /*
     * Parse the command line.
     */
    if (argc == 2 && argv[1])
	def_listname = lowercase(argv[1]);

    /*
     * Parse header for "from".
     */
    while (fgets(buf, BUFSIZE - 1, stdin) && buf[0] != '\n') {
	/* drop trailing newline */
	buf[strlen(buf) - 1] = '\0';

	/* get the From: line so we can return the answer */
	if (!strncasecmp(buf, "From:", 5)) {
	    if ((from = xstrdup(&buf[5]))) {
		ParseAddress(from);
	    }
	}
	else if (!strncasecmp(buf, "Reply-To:", 9))
	  /* get the Reply-to: line, which overrides From: */
	  if ((replyto = xstrdup(&buf[9])))
	    ParseAddress(replyto);
    }

    /*
     * Check "from" validity!
     */
    if (!from) {
	log(LOG_ERR, "Received mail without 'From:'-line!");
	return 5;
    }
    if (replyto) {
	free(from);
	from = replyto;		/* use Reply-To address over From */
    }

    if (IsBanned(from, NULL)) {
	log(LOG_INFO, BANNED_MSG, from);
	sprintf(tmpbuf, "%s -f " PETIDOMOMANAGER " -t", SENDMAIL);
	if ((mailer = popen(tmpbuf, MODE_PIPETO))) {
	    fprintf(mailer, "From: daemon@%s.%s (petidomo address banning service)\n", NODENAME, DOMAINNAME);
	    fprintf(mailer, "To: petidomo-manager@%s.%s\n", NODENAME, DOMAINNAME);
	    fprintf(mailer, "Subject: Received mail from banned address\n\n");
	    fprintf(mailer, "Address '%s' is banned from petidomo.\n\n", from);
	    while (fgets(buf, BUFSIZE - 1, stdin)) {
		fprintf(mailer, " | %s", buf);
	    }
	}
	else
	    logerrno(LOG_ERR, "popen(%s) failed", tmpbuf);
	return 0;
    }

    /*
     * Process mail body.
     */
    while (fgets(buf, BUFSIZE - 1, stdin)) {
	if (buf[strlen(buf) - 1] == '\n')
	    buf[strlen(buf) - 1] = '\0';	/* drop trailing \n */

	if (!(*buf))		/* check for empty lines */
	    continue;

	lowercase(buf);		/* turn string into lowercase */

	log(LOG_INFO, "%s: %s", from, buf);	/* log the request */

	if (isKeyword(buf, "passwd")) {
	    gotcommand++;
	    sscanf(buf, "%*s%s", passwd);
	    continue;
	}

	if (isKeyword(buf, "list")
	    || isKeyword(buf, "delete-all")
	    || isKeyword(buf, "unsubscribe-all")) {
	    gotcommand++;
	    listsearch(from, buf);
	    continue;
	}

	if (isKeyword(buf, "add")
	    || isKeyword(buf, "subscribe")) {
	    gotcommand++;
	    subscription(from, buf, 1);
	    continue;
	}

	if (isKeyword(buf, "delete")
	    || isKeyword(buf, "del")
	    || isKeyword(buf, "unsubscribe")) {
	    gotcommand++;
	    subscription(from, buf, 0);
	    continue;
	}

	if (isKeyword(buf, "index")
	    || isKeyword(buf, "longindex")) {
	    gotcommand++;
	    sendindex(from, buf, !strcasecmp(buf, "longindex"));
	    continue;
	}

	if (isKeyword(buf, "faq")) {
	    gotcommand++;
	    sendfaq(from, buf);
	    continue;
	}

	if (isKeyword(buf, "help")) {
	    gotcommand++;
	    sendhelp(from, buf);
	    continue;
	}

	/* flush message if > 5 lines of garbage */
	if (++garbage > 5) {
	    log(LOG_ERR, "Garbled message, flushing...");
	    break;
	}
    }

    if (!gotcommand) {
	log(LOG_ERR, "%s: indecipherable mail --> sending HELP back", from);
	sendhelp(from, "indecipherable");
    }

    return 0;
}

void
callmailer(char *carboncopy, char *toaddr, char *request)
{
    sprintf(tmpbuf, "%s -f " PETIDOMOMANAGER " -F \"Mailing List Processor\" -t", SENDMAIL);
    if (!(mailer = popen(tmpbuf, MODE_PIPETO))) {
	logerrno(LOG_ERR, "popen(%s) failed", tmpbuf);
	exit(10);
    }
    if (def_listname)
	fprintf(mailer, "Reply-To: %s-request@", def_listname);
    else
	fprintf(mailer, "Reply-To: " PETIDOMOADDR "@");
    fprintf(mailer, "%s.%s\n", NODENAME, DOMAINNAME);
    fprintf(mailer, "Precedence: junk\n");
    fprintf(mailer, "To: %s\n", toaddr);
    if (carboncopy && *carboncopy)
	fprintf(mailer, "Cc: %s\n", carboncopy);
    fprintf(mailer, "Subject: Re: your petidomo request \"%s\"\n", request);
    fprintf(mailer, "X-Version: " VERS " (http://home.pages.de/~simons/software/)\n");
    fprintf(mailer, "\n");
#ifdef BETA
    fprintf(mailer, BETA_MSG);
#endif
    /*
     * Here I would like to take the time to greet Walter 'wolli'
     * Steiner, the man who suggested that mail from the daemon
     * should be classified as junk. Thanks wolli!!
     */
}

void
mailcat(char *mfname, char *prefix)
{
    FILE     *mf;

    if (!(mf = fopen(mfname, "r"))) {
	logerrno(LOG_ERR, "open(%s) failed", mfname);
	fprintf(mailer, "This should have been the contents of\n");
	fprintf(mailer, "file '%s' but it's missing!\n", mfname);
	return;
    }

    /* copy the message file into the mailer */
    while (fgets(tmpbuf, BUFSIZE, mf)) {
	if (prefix && *prefix)
	    fputs(prefix, mailer);
	fputs(tmpbuf, mailer);
    }
    fclose(mf);
    return;
}

void
ParseAddress(char *line)
{
    int       brackets,
              start,
              i,
              z;

    for (start = 0; isspace(line[start]); start++) ;

    for (i = start, z = 0, brackets = 0; line[i] != '\0'; i++) {
	if (line[i] == '(')
	    brackets++;
	else if (line[i] == ')')
	    brackets--;
	else if (line[i] == '<' && brackets == 0) {
	    i++;
	    while (1) {		/* Forever! */
		if (line[i] == '>' &&
		    (line[i + 1] == '\0' || isspace(line[i + 1])))
		    break;
		line[z++] = line[i++];
	    }
	    line[z] = '\0';
	    break;
	}
    }
    if (z == 0) {		/* No <address> occured. */
	for (i = start; line[i] && !isspace(line[i]);)
	    line[z++] = line[i++];
	line[z] = '\0';
    }
}

int
isKeyword(char *line, char *keyword)
{
    int       len = strlen(keyword);

    if (strncasecmp(line, keyword, len))
	return 0;
    if (isspace(line[len]) || line[len] == '\0')
	return 1;

    return 0;
}
