/*
 *      $Filename: subscribe.c $
 *      $Revision: 3.25 $
 *      $Date: 1996/12/27 14:38:05 $
 *
 *      Copyright (C) 1993 by Brian Kantor <brian@nothing.ucsd.edu>
 *      Copyright (C) 1994-96 by Peter Simons <simons@peti.rhein.de>
 *
 *      This program is free software; you can redistribute it and/or
 *      modify it under the terms of the GNU General Public License as
 *      published by the Free Software Foundation; either version 2 of
 *      the License, or (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *      General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *      $Id: subscribe.c,v 3.25 1996/12/27 14:38:05 simons Exp $
 *
 */

/************************************* includes ***********/
#include "petidomo.h"

/************************************* global variables ***/
static char rcsid[] = "$Id: subscribe.c,v 3.25 1996/12/27 14:38:05 simons Exp $";

/************************************* program ************/
int
subscription(char *from, char *command, int add)
{
    FILE *    fh;
    FILE     *list;
    FILE     *listtmp;
    char      grp[256],
              adr[256],
              tmp[256],
              tmp2[256],
              buf[BUFSIZ];
    int       del = 0,
              found = 0,
              i;

    i = sscanf(command, "%s%s%s", tmp, adr, grp);
    debug(2, "subscription: i = %d", i);
    switch(i) {

      /* Add without parameters. */
      case 1:
	  if (def_listname) {
	      /* Do we have a default list name? Then use it. */
	      strcpy(grp, def_listname);
	      strcpy(adr, from);
	  }
	  else {
	      /* Syntax error */
	      sendhelp(from, command);
	      return -1;
	  }
	  break;

      /* Add with only one parameter. */
      case 2:
	  if (def_listname)
	    /* If default list is available, interpret parameter as address. */
	    strcpy(grp, def_listname);
	  else {
	      /* If no default list has been provided, use parameter
		 as list's name and From: as address. */
	      strcpy(grp, adr);
	      strcpy(adr, from);
	  }
	  break;

      /* Add with two parameters. */
      case 3:
	  /* Use first parameter as address and second as list's name. */
	  break;

      default:
	  sendhelp(from, command);
	  return -1;
    }
    debug(2, "adr = %s, grp = %s", adr, grp);

    /* Do sanity check. */

    if (!(strstr(adr, "!") || strstr(adr, "@"))) {
	/* Hmm, address does not contain a 'bang' nor an 'at'? */

	if (strstr(grp, "!") || strstr(grp, "@")) {
	    /* But listname does? Then we swap them. */
	    log(LOG_NOTICE, "List's name and address seem to be mixed up. I'll swap them.");
	    strcpy(tmp, adr);
	    strcpy(adr, grp);
	    strcpy(grp, tmp);
	}
	else {
	    log(LOG_NOTICE, "The address '%s' does not contain a 'bang' or 'at' and will "\
		"be rejected. Please don't use local addresses.", adr);
	    sendhelp(from, "indecipherable");
	    return -1;
	}
    }

    debug(2, "adr = %s, grp = %s", adr, grp);

    if (!strcasecmp(grp, adr)) {
	sendhelp(from, command);
	return -1;
    }

    if (IsBanned(adr, NULL) || IsBanned(adr, grp)) {
	log(LOG_NOTICE, BANNED_MSG, adr);
	return 0;
    }

    if (!(authen(grp, passwd))) {
	log(LOG_NOTICE, "%s tried to %s an address but couldn't provide the correct password", adr, (add) ? "add" : "delete");
	callmailer("", from, command);
	fprintf(mailer, "You failed to provide the correct password. Your\n");
	fprintf(mailer, "command has not been carried out, I'm afraid.\n");
	pclose(mailer);
	return 0;
    }

    sprintf(tmp, "%s/%s/list", LISTDIR, grp);
    if ((fh = fopen(tmp, "r")) != NULL)
      fclose(fh);
    if (fh == NULL) {
	logerrno(LOG_NOTICE, "open(%s) failed", tmp);
	callmailer("", from, command);
	fprintf(mailer, "The mailing list \"%s\" could not be found.\n", grp);
	fprintf(mailer, "You may use the INDEX command to get a listing\n");
	fprintf(mailer, "of available mailing lists.\n");
	pclose(mailer);
	return (-1);
    }

    if (add) {
	if (!(list = fopen(tmp, "r+"))) {
	    logerrno(LOG_ERR, "open(%s) failed", tmp);
	    callmailer(PETIDOMOMANAGER, from, command);
	    fprintf(mailer, FATALERROR_MSG, 1);
	    pclose(mailer);
	    return (-1);
	}

	/* check if address is already subscribed */
	while (fgets(buf, sizeof(buf), list)) {
	    buf[strlen(buf) - 1] = '\0';
	    if (!strcasecmp(buf, adr))
		found++;
	}
	if (!found)
	    if (fseek(list, 0L, SEEK_END) != -1)
		fprintf(list, "%s\n", adr);
	fclose(list);
    }
    else {
	del = 0;
	if (!(list = fopen(tmp, "r"))) {
	    logerrno(LOG_ERR, "open(%s) failed", tmp);
	    callmailer(PETIDOMOMANAGER, from, command);
	    fprintf(mailer, FATALERROR_MSG, 2);
	    pclose(mailer);
	    return (-1);
	}

	sprintf(tmp2, "%s.tmp", tmp);
	if (!(listtmp = fopen(tmp2, "w"))) {
	    logerrno(LOG_ERR, "open(%s) failed", tmp2);
	    callmailer(PETIDOMOMANAGER, from, command);
	    fprintf(mailer, FATALERROR_MSG, 3);
	    pclose(mailer);
	    return (-1);
	}
	/* copy the list, omitting the one address */
	while (fgets(buf, sizeof(buf), list)) {
	    buf[strlen(buf) - 1] = '\0';
	    if (strcasecmp(buf, adr)) {
		fputs(buf, listtmp);
		fputs("\n", listtmp);
	    }
	    else
		del++;
	}
	fclose(listtmp);
	fclose(list);
	/* replace the old list with the shortened one */

	remove(tmp);		/* delete original file */
	rename(tmp2, tmp);	/* put updated one in place */
    }

    /* Mail subscription confirmation and introduction files. */

    sprintf(tmp, "owner-%s", grp);
    if (strcasecmp(from, adr)) {
	callmailer(tmp, adr, command);
	fprintf(mailer, "Per request by %s\n", from);
    }
    else {
	callmailer(tmp, from, command);
	fprintf(mailer, "Per your request\n");
    }

    fprintf(mailer, "\t\"%s\"\n", command);

    if (add)
	if (!found) {
	    fprintf(mailer, "'%s' was ADDED to the '%s' mailing list.\n", adr, grp);
	    sprintf(buf, "%s/%s/introduction", LISTDIR, grp);
	    if ((fh = fopen(buf, "r")) != NULL)
	      fclose(fh);
	    if (fh != NULL) {
		fprintf(mailer, "\n\n------------------------ List introduction ---------------------------\n");
		mailcat(buf, "");
		fprintf(mailer, "----------------------------------------------------------------------\n");
	    }
	    else {
		logerrno(LOG_WARNING, "open(%s) failed", buf);
	    }
	}
	else
	    fprintf(mailer, "'%s' is already subscribed to the '%s' mailing list.\n", adr, grp);
    else if (del) {
	fprintf(mailer, "'%s' was DELETED from the '%s' mailing list.\n\n", adr, grp);
	fprintf(mailer, DELETED_MSG);
    }
    else
	fprintf(mailer, "'%s' was NOT FOUND on the '%s' mailing list.\n", adr, grp);

    pclose(mailer);

    return 0;
}
